!----------------------------------------------------------------------
!  Set up lower boundary (LB) conditions for GSK region by passing 
!  vlaues from FLIP
!----------------------------------------------------------------------
subroutine backgsk(jbg,ns)
    use inparm
    use gskparm

    integer ns,jbg
    real(8) N,TI,F,U,BG,BM,GR,R,SL !,XIONN,XIONV,XMASS
    real(8) OPLS,VOPLS,HE,UE,HEPRD,OP
    
    COMMON/FJS/N(4,401),TI(3,401),F(20)
    COMMON/VN/U(2,401),BG(401),BM(401),GR(2,401),R(2,401),SL(401)
!    COMMON/MINOR/XIONN(9,401),XIONV(9,401),XMASS(9)
    COMMON/A/OPLS(401),VOPLS(401),HE(401),UE(2,401),HEPRD(401),OP(401)

!******************************************************************
!     Ti(3,1:401) = electron temperature
!     Ti(1,1:401) = O+ temperature (K)
!     Ti(2,1:401) = H+ temperature (K)
!     N(1,1:401)  = O+ density  (cm^-3)
!     N(2,1:401)  = H+ density  (cm^-3)
!     N(4,1:401)  = He+ density  (cm^-3)
!     U(1,1:401)  = O+ velocity (cm/s)
!     U(2,1:401)  = H+ velocity (cm/s)
!     UE(1,1:401)  = He+ velocity (cm/s)

!     FheatO = O+ heat flux [erg/(cm^2 s)]
!     FheatH = H+ heat flux [erg/(cm^2 s)]
!     FheatE = electron heat flux [erg/(cm^2 s)]
!     FehtO  = energy loss rate from electrons to O+ [erg/(cm^3 s)]
!     FehtH  = energy loss rate from electrons to H+ [erg/(cm^3 s)]
!     FehtHe = energy loss rate from electrons to He+ [erg/(cm^3 s)]
!     FEEHT  = electron heating rate [erg/(cm^3 s)]

!  Velcoities and heat flows are directed from north to south if
!  positive in FLIP, but positive if upward in GSK
!******************************************************************

    Fden_O(ns) = N(1,jbg)
    Fden_H(ns) = N(2,jbg)
    Fden_He(ns)= N(4,jbg)
    FV_O(ns)   = U(1,jbg)*1.0d-2   ! changed to m/s
    FV_H(ns)   = U(2,jbg)*1.0d-2   ! changed to m/s
    FV_He(ns)  = UE(1,jbg)*1.0d-2  ! changed to m/s
    FT_O(ns)   = TI(1,jbg)
    FT_H(ns)   = TI(2,jbg)
    FT_He(ns)  = TI(2,jbg)        !Take H+ tem as He+ tem
    FT_e(ns)   = TI(3,jbg)

! electron heat flux (eqs.(5.141) & (5.146) in Ionospheres Physics,
! Plasma Physics, and Chemistry by R.W. Schunk & A. F. Nagy)
    FheatE(ns)=-1.232d-6*TI(3,jbg)**2.5d0*(TI(3,jbg+1)-TI(3,jbg-1)) &
                 /(SL(jbg+1)-SL(jbg-1))
end subroutine
!**********************************************************************
subroutine num_inject(jg,ns)
    use inparm
    use gskparm

    integer jg,ns
    character :: text*16
    real(8) M_O,Re,ak,pi,M_H,M_He,vzmax,vfO,vfH,vfHe

    data M_O/2.65676233d-26/,M_H/1.67382304d-27/,M_He/6.69529216d-27/
    data Re/6371.2d3/,ak/1.38065d-23/,pi/3.1415926d0/

    real(8), external :: ERRORF

    call backgsk(jg,ns)

!  perpendicular oxygen/Hydrog thermal velocity (in Re/s)
    VtppO(ns)=sqrt(2.0d0*ak*FT_O(ns)/M_O)/Re
    VtppH(ns)=sqrt(2.0d0*ak*FT_H(ns)/M_H)/Re
    VtppHe(ns)=sqrt(2.0d0*ak*FT_He(ns)/M_He)/Re

!  parallel thermal velocity (in Re/s)
    VtprO(ns)=VtppO(ns)
    VtprH(ns)=VtppH(ns)
    VtprHe(ns)=VtppHe(ns)

!  Convert O+ and H+ bulk velocity of injected distribution to Re/s
    FV_O(ns)=FV_O(ns)/Re
    FV_H(ns)=FV_H(ns)/Re
    FV_He(ns)=FV_He(ns)/Re

!-------------
!  factorO(H): maxm parallel velocity (relative to average velocity)
!              allowed for injected particles
!  vaxmaxO(H): parallel velocity at vpar*f(vpar) = maxm
!  fmaxO(H): maxm value of vpar*f(vpar)
!-------------
    factorO(ns)=VtprO(ns)*10.0d0
    vzmax=(FV_O(ns)+sqrt(FV_O(ns)**2+2.0d0*VtprO(ns)**2))/2.0d0
    fmaxO(ns)=vzmax*exp(-((vzmax-FV_O(ns))/VtprO(ns))**2)

    factorH(ns)=VtprH(ns)*5.0d0
    vzmax=(FV_H(ns)+sqrt(FV_H(ns)**2+2.0d0*VtprH(ns)**2))/2.0d0
    fmaxH(ns)=vzmax*exp(-((vzmax-FV_H(ns))/VtprH(ns))**2)

    factorHe(ns)=VtprHe(ns)*5.0d0
    vzmax=(FV_He(ns)+sqrt(FV_He(ns)**2+2.0d0*VtprHe(ns)**2))/2.0d0
    fmaxHe(ns)=vzmax*exp(-((vzmax-FV_He(ns))/VtprHe(ns))**2)

    vfO=0.5d0*(VtprO(ns)*exp(-(FV_O(ns)/VtprO(ns))**2)/sqrt(pi)+ &
        FV_O(ns)*(1.0d0+ERRORF(FV_O(ns)/VtprO(ns))))
    vfH=0.5d0*(VtprH(ns)*exp(-(FV_H(ns)/VtprH(ns))**2)/sqrt(pi)+ &
        FV_H(ns)*(1.0d0+ERRORF(FV_H(ns)/VtprH(ns))))
    vfHe=0.5d0*(VtprHe(ns)*exp(-(FV_He(ns)/VtprHe(ns))**2)/sqrt(pi)+ &
        FV_He(ns)*(1.0d0+ERRORF(FV_He(ns)/VtprHe(ns))))

! area in m^-2, Re in m, density in cm^-3
    bottom_upO(ns)=int(1.0e6*Fden_O(ns)/rep_O*(vfO*Re)*area(1)*dtg)
    bottom_upH(ns)=int(1.0e6*Fden_H(ns)/rep_H*(vfH*Re)*area(1)*dtg)
    bottom_upHe(ns)=int(1.0e6*Fden_He(ns)/rep_He*(vfHe*Re)*area(1)*dtg)

    write(*,'(3(A,i5))') ' bottom_upO = ', int(bottom_upO(ns)), &
       ' bottom_upH = ', int(bottom_upH(ns)),' bottom_upHe = ', &
       int(bottom_upHe(ns))
    write(*,*) 'VtprO = ',VtprO(ns)*Re,' VtprH = ',VtprH(ns)*Re,' m/s', &
               ' VtprHe = ',VtprHe(ns)*Re,' m/s'

    if(bottom_upO(ns).gt.200000) then
        write(*,*) 'Too many injected O+. Decrease ndO'
        write(63,'(a)') 'Too many injected O+. Decrease ndO'
        write(63,'(a)') 'FV_O, VtprO, Fden_O ',FV_O*Re,VtprO*re,Fden_O
        write(63,'(a)') 'Shutdown in BCKGSK'
        stop
    endif
!    if(bottom_upO(ns).eq.0) then
!      write(*,*) 'bottom_upO=0. Inecrease ndO'
!      stop
!    endif
    if(bottom_upH(ns).gt.5000) then
        write(*,*) 'Too many injected H+. Decrease ndH '
        write(63,'(a)') 'Too many injected H+. Decrease nHO'
        write(63,'(a)') 'FV_H, VtprH, Fden_H ',FV_H*Re,VtprH*re,Fden_H
        write(63,'(a)') 'Shutdown in BCKGSK'
        stop
    endif
!    if(bottom_upH(ns).eq.0) then
!      write(*,*) 'bottom_upH=0. Inecrease ndH'
!      stop
!    endif
    if(bottom_upHe(ns).gt.5000) then
        write(*,*) 'Too many injected He+. Decrease ndHe'
        write(63,'(a)') 'Too many injected He+. Decrease ndO'
        write(63,'(a)') 'FV_He, VtprHe, Fden_He ',FV_He*Re,VtprHe*re,Fden_He
        write(63,'(a)') 'Shutdown in BCKGSK'
        stop
    endif
!    if(bottom_upHe(ns).eq.0) then
!      write(*,*) 'bottom_upHe=0. Inecrease ndHe'
!      stop
!    endif
end subroutine
!**********************************************************************
subroutine inject_s(i,inew,dt,factor,fmax,FV,Vtpr,Vtpp,imax,OHgen)
    use gskparm

    integer :: inew, imax, i, j

    real(8), dimension(np, 3) :: OHgen

    real(8) :: dt, factor, fmax, Vtpr, Vtpp, FV, FVa
    real(8) :: s, vs, vp, ft, fs, s0, dtm, ran2

    dtm=dt/float(inew)
    s0=sg(i)
    FVa=FV
 
    if(i.eq.num+1) FVa=-FV

    do j=1,inew
        imax=imax+1
!------------------------------------------------------------------
!---New by Y.J.Su on 8/15/95
!   We need to check if bottom cell particle is within distribution
!------------------------------------------------------------------
!   injected ptcl vs>0(<0) at i=1(num). FV positive from N->S
11      vs=ran2(iseed)*factor
        ft=ran2(iseed)*fmax
        fs=vs*exp(-((vs-FVa)/Vtpr)**2)
        if (ft.gt.fs) go to 11

        vp=Vtpp*sqrt(-log(1.0d0-ran2(iseed)))

        if(i.eq.num+1) vs=-vs  
        s=s0+vs*(float(j)*dtm)

        OHgen(imax,1)=s
        OHgen(imax,2)=vs
        OHgen(imax,3)=vp
    enddo
end subroutine
