!-----------------------------------------------------------------------
! Calculate electric field and potential
!
! XYWU: 11/11/99 modified to incorporate hot magnetospheric plasma
! Rewritten by Jiannan Tu  Feb 20, 2003
!-----------------------------------------------------------------------
subroutine efdfind(efield,poten)
    use gskparm

    real(8) Te(nc),efield(nc),poten(nc)
    real(8) S1,Re,qme,kbe,ds
    real(8) termO,termH,termHe,terme,term_p,term_f,term_a,term_j
    integer i

    data Re/6371.2e+3/,qme/1.758842514d11/,kbe/8.617525902d-5/

    Te(1:num+1)=(tpr_e(1:num+1)+2.0d0*tpp_e(1:num+1))/3.0d0

! The equation for efd is valid in either CGS or SI unit. Here use SI
! Gravitational term = 0 because of neutrality and all speices
! are single charged.
    do i=2,num
        if(den_e(i).le.0.0.and.den_hote(i).le.0.0) then
            efield(i)=0.0d0
            cycle
        endif

        ! ds in m
        ds=(sg(i)-sg(i-1))*Re

        if(eqe.eq.0) then
            efield(i)=-kbe*(Te(i)/den_e(i)*(den_e(i)-den_e(i-1))-(Te(i)-Te(i-1)))/ds
            cycle
        endif

        S1=area(i)*(den_e(i)+den_hote(i))*qme

! fluid term: (length in m; area in m^2; density in cm^-3; Re in m)
! Note area*den in both numerator and denominator has the same units
        termO=den_O(i)*vel_O(i)*vel_O(i)*area(i)- &
              den_O(i-1)*vel_O(i-1)*vel_O(i-1)*area(i-1)
        termH=den_H(i)*vel_H(i)*vel_H(i)*area(i)- &
              den_H(i-1)*vel_H(i-1)*vel_H(i-1)*area(i-1)
        termHe=den_He(i)*vel_He(i)*vel_He(i)*area(i)- &
               den_He(i-1)*vel_He(i-1)*vel_He(i-1)*area(i-1)
        terme=den_e(i)*vel_e(i)*vel_e(i)*area(i)- &
              den_e(i-1)*vel_e(i-1)*vel_e(i-1)*area(i-1)

        term_f=(termO+termH+termHe-terme)/ds

! pressure gradient term: (keep only electron term; others too small)
        term_p=(den_e(i)*tpr_e(i)-den_e(i-1)*tpr_e(i-1)+ &
                den_hote(i)*tpr_hote(i)-den_hote(i-1)* &
                tpr_hote(i-1))/(den_e(i)*ds)

! temperature anisotropic term: (keep only electron term)
        term_a=((tpr_e(i)-tpp_e(i))+(tpr_hote(i)-tpp_hote(i)))*dA(i)

! time variation of current (in eq J=A*sum(qs*ns*vs) is current)
        term_j=0.0d0 !(=dJ/dt/e)

! e-field (V/m)
        efield(i)=(term_j+term_f)/S1-kbe*(term_p+term_a)
    enddo

    call smoothp(efield(2:num+1),num-1)
!    call LGRG(sg(2:6),efield(2:6),5,sg(1),efield(1))
!    call LGRG(sg(num-4:num),efield(num-4:num),5,sg(num+1),efield(num+1))

    efield(1)=efield(2)
    efield(num+1)=efield(num)

    poten(1)=0.0d0
    do i=2,num+1
        poten(i)=poten(i-1)-(efield(i)+efield(i-1))*(sg(i)-sg(i-1))
    enddo

end subroutine efdfind

