!-------------------------------------------------------------------------
!  Liouville Mapping:  (hot magnetospheric plasmas) X.Y.Wu on 11/11/99
!  Documented and modified: Jiannan Tu on Feb 13, 2003
!
!   Input:  n        - number of grid cells
!           rr       - geocentrical distance of grid cells (in Re)
!           poten    - parallel potential (V)
!
!   Output: den_p    - magnetos H+ density at current time step (cm^-3)
!           tpp_p    - magnetos H+ perp temp at current time step (K)
!           tpr_p    - magnetos H+ parallel temp at current time step (K)
!           tpp_hote - magnetos ele perp temp at current time step (K)
!           tpr_hote - magnetos ele parallel temp at current time step (K)
!
!  Rewitten by Jiannan Tu, Feb 13, 2003
!-------------------------------------------------------------------------
subroutine Liouville(n,poten)
     use gskparm

     integer n,i

     real(8) poten(nc),ratio_p,ratio_e,ratio_B,kb,q

     data kb,q /1.38065d-23, 1.60219d-19/

     ratio_p = tpp_p(n)/tpr_p(n)
     ratio_e = tpp_hote(n)/tpr_hote(n)

     do i=1,n-1
         ratio_B = ((rr(i)+rr(i+1))/(rr(n)+rr(n+1)))**3
         den_p(i) = den_p(n)*exp(-q*(poten(i)-poten(n))/(kb*tpr_p(n)))/ &
     	           (ratio_p + (1.-ratio_p)*ratio_B)

         tpp_p(i) = tpp_p(n)/((1.0d0-ratio_B)*ratio_p + ratio_B)
         tpr_p(i) = tpr_p(n)
         tpp_hote(i) = tpp_hote(n)/((1.0d0-ratio_B)*ratio_e + ratio_B)
         tpr_hote(i) = tpr_hote(n)
     enddo

end subroutine Liouville
