!     
! File:   MDyFk.f90
! Author: wangy
!
! Created on August 6, 2010, 7:34 AM
! Paralleled computing by MPI for DyFk model


! The program was modified to run flip and Gsk based on continue run. (numtasks=2).
! In subroutine Flip, I comment line 3682.
! In the main program, I put the MPI_BCAST for nodes which are not master to the subroutine 'tgsk_mp.f90',
! since some variables are not defined in the main program.
program main
    use inparm
    use gskparm
    include 'mpif.h'

    integer rc,numtasks,master

    integer rank,ierr,status(MPI_STATUS_SIZE)
    common /mpi/rank,ierr,status

    integer nst,err0
    integer i
    DOUBLE PRECISION :: starttime,endtime

     master=0

!******************** MPI environment management***********************
    call MPI_INIT(ierr)
    if(ierr.ne.MPI_SUCCESS) then
        print *, 'Error starting MPI program. Terminating.'
        call MPI_ABORT(MPI_COMM_WORLD, rc, ierr)
    endif

    call MPI_COMM_RANK(MPI_COMM_WORLD, rank, ierr)
    call MPI_COMM_SIZE(MPI_COMM_WORLD, numtasks, ierr)

    print *, 'numtasks=', numtasks
    print *,'rank=',rank, 'num of rank', numtasks

    starttime=MPI_Wtime()

!******************* Initialize the array of particles*****************

    Oxygen(1:np,1)=999.0d0; Oxygen(1:np,2)=0.0d0; Oxygen(1:np,3)=0.0d0
    Hydrog(1:np,1)=999.0d0; Hydrog(1:np,2)=0.0d0; Hydrog(1:np,3)=0.0d0
    Helium(1:np,1)=999.0d0; Helium(1:np,2)=0.0d0; Helium(1:np,3)=0.0d0

    err0=0
     
!******************* Master read parameters ****************************
    if(rank.eq.master) then
		   
       CALL READ_PARAM()

       CALL OPEN_FILES()

       IF(GSKR=='Yes') CALL READ_GSK_PARM(err0)
       IF(err0 /=0) STOP
       print *, '****** rank=master finished here ******'
    endif

!***************** Broadcast parameters *********************************

    call MPI_BARRIER(MPI_COMM_WORLD,ierr)
    call MPI_BCAST(GSKR,3,MPI_CHARACTER,0,MPI_COMM_WORLD,ierr)
    call MPI_BCAST(DRIFT,3,MPI_CHARACTER,0,MPI_COMM_WORLD,ierr)
    call MPI_BCAST(GSK2FILP,3,MPI_CHARACTER,0,MPI_COMM_WORLD,ierr)
    call MPI_BCAST(CRUN,3,MPI_CHARACTER,0,MPI_COMM_WORLD,ierr)
    call MPI_BCAST(NAURON,1,MPI_INTEGER,0,MPI_COMM_WORLD,ierr)
    call MPI_BCAST(NAUROF,1,MPI_INTEGER,0,MPI_COMM_WORLD,ierr)
    call MPI_BCAST(NPYNON,1,MPI_INTEGER,0,MPI_COMM_WORLD,ierr)
    call MPI_BCAST(NPYNOF,1,MPI_INTEGER,0,MPI_COMM_WORLD,ierr)
    call MPI_BCAST(WAVEON,1,MPI_INTEGER,0,MPI_COMM_WORLD,ierr)
    call MPI_BCAST(WAVEOF,1,MPI_INTEGER,0,MPI_COMM_WORLD,ierr)
    call MPI_BCAST(HOTPON,1,MPI_INTEGER,0,MPI_COMM_WORLD,ierr)
    call MPI_BCAST(HOTPOF,1,MPI_INTEGER,0,MPI_COMM_WORLD,ierr)
    call MPI_BCAST(CURON,1,MPI_INTEGER,0,MPI_COMM_WORLD,ierr)
    call MPI_BCAST(CUROF,1,MPI_INTEGER,0,MPI_COMM_WORLD,ierr)
    call MPI_BCAST(nstep,1,MPI_INTEGER,0,MPI_COMM_WORLD,ierr)
    call MPI_BCAST(iout,1,MPI_INTEGER,0,MPI_COMM_WORLD,ierr)

    call MPI_BCAST(err0,1,MPI_INTEGER,0,MPI_COMM_WORLD,ierr)
    call MPI_BCAST(num,1,MPI_INTEGER,0,MPI_COMM_WORLD,ierr)
    call MPI_BCAST(loopmax,1,MPI_INTEGER,0,MPI_COMM_WORLD,ierr)
    call MPI_BCAST(iseed,1,MPI_INTEGER,0,MPI_COMM_WORLD,ierr)
    call MPI_BCAST(eqe,1,MPI_INTEGER,0,MPI_COMM_WORLD,ierr)
    call MPI_BCAST(zb,1,MPI_REAL8,0,MPI_COMM_WORLD,ierr)
    call MPI_BCAST(scalg,1,MPI_REAL8,0,MPI_COMM_WORLD,ierr)
    call MPI_BCAST(dtg,1,MPI_REAL8,0,MPI_COMM_WORLD,ierr)
    call MPI_BCAST(ndO,1,MPI_INTEGER,0,MPI_COMM_WORLD,ierr)
    call MPI_BCAST(ndH,1,MPI_INTEGER,0,MPI_COMM_WORLD,ierr)
    call MPI_BCAST(ndHe,1,MPI_INTEGER,0,MPI_COMM_WORLD,ierr)
    call MPI_BCAST(vsminO,1,MPI_REAL8,0,MPI_COMM_WORLD,ierr)
    call MPI_BCAST(vsminH,1,MPI_REAL8,0,MPI_COMM_WORLD,ierr)
    call MPI_BCAST(vsminHe,1,MPI_REAL8,0,MPI_COMM_WORLD,ierr)
    call MPI_BCAST(noss,1,MPI_INTEGER,0,MPI_COMM_WORLD,ierr)
    call MPI_BCAST(nosp,1,MPI_INTEGER,0,MPI_COMM_WORLD,ierr)
    call MPI_BCAST(nhss,1,MPI_INTEGER,0,MPI_COMM_WORLD,ierr)
    call MPI_BCAST(nhsp,1,MPI_INTEGER,0,MPI_COMM_WORLD,ierr)
    call MPI_BCAST(nhess,1,MPI_INTEGER,0,MPI_COMM_WORLD,ierr)
    call MPI_BCAST(nhesp,1,MPI_INTEGER,0,MPI_COMM_WORLD,ierr)
    call MPI_BCAST(dvso,1,MPI_REAL8,0,MPI_COMM_WORLD,ierr)
    call MPI_BCAST(dvpo,1,MPI_REAL8,0,MPI_COMM_WORLD,ierr)
    call MPI_BCAST(dvsh,1,MPI_REAL8,0,MPI_COMM_WORLD,ierr)
    call MPI_BCAST(dvph,1,MPI_REAL8,0,MPI_COMM_WORLD,ierr)
    call MPI_BCAST(dvshe,1,MPI_REAL8,0,MPI_COMM_WORLD,ierr)
    call MPI_BCAST(dvphe,1,MPI_REAL8,0,MPI_COMM_WORLD,ierr)
    call MPI_BCAST(alpha,1,MPI_REAL8,0,MPI_COMM_WORLD,ierr)
    call MPI_BCAST(rh0,2,MPI_REAL8,0,MPI_COMM_WORLD,ierr)
    call MPI_BCAST(rhscl,2,MPI_REAL8,0,MPI_COMM_WORLD,ierr)

    print *, 'FLIP PARAMETER BROADCAST FINISHED', 'RANK=', rank,'loopmax=',loopmax

!******************* Start to Run DyFk program ************************

    if(rank.eq.0) then
       CALL FLIP(nst)
       if(nst==nstep) then
          CLOSE(16)
          CLOSE(4)
          CLOSE(66)
          CLOSE(67)
          CLOSE(17)
          CLOSE(18)
          CLOSE(46)
          CLOSE(63)
       endif
    else
       do i=1,nstep

          print *, 'step=',i,'rank=',rank,'start to run GSK'
   
          call tgsk_mp()

       enddo
    endif
    
    
    print *,'rank=',rank,'finished'
    CALL MPI_BARRIER(MPI_COMM_WORLD,ierr)
    endtime=MPI_Wtime()
    if(rank.eq.0) print *, 'it took t=', endtime-startime, 'seconds'

    call MPI_FINALIZE(ierr)

End Program main


!***********************************************************************

SUBROUTINE OPEN_FILES()
    USE INPARM
    !INTEGER :: LNBLNK

    OPEN(16,FILE='/home/wangy/DyFk/DYFKRUN.LOG',STATUS='UNKNOWN')
    OPEN(4,FILE='/home/wangy/DyFk/PLOT/EQUATOR.DAT',STATUS='UNKNOWN')
    OPEN(66,FILE='/home/wangy/DyFk/PLOT/VEXB.DAT',STATUS='UNKNOWN')
    OPEN(67,FILE='/home/wangy/DyFk/PLOT/TOTCONT.DAT',STATUS='UNKNOWN')
    OPEN(17,FILE='/home/wangy/DyFk/PLOT/GRIDFLIP.DAT',STATUS='UNKNOWN')
    OPEN(18,FILE='/home/wangy/DyFk/PLOT/TRANSALT.DAT',STATUS='UNKNOWN')

END SUBROUTINE

!***********************************************************************

SUBROUTINE READ_GSK_PARM(err0)
    USE INPARM
    USE GSKPARM

    INTEGER :: i, j, err0

    OPEN(30,FILE='/home/wangy/DyFk/GSKRUN.DDD',ERR=35)

    READ(30,*,ERR=35,END=35) num, loopmax, iseed, eqe
    READ(30,*,ERR=35,END=35) zb, scalg, dtg
    READ(30,*,ERR=35,END=35) ndO, ndH, ndHe
    READ(30,*,ERR=35,END=35) vsminO, vsminH, vsminHe
    READ(30,*,ERR=35,END=35) noss,nosp,nhss,nhsp,nhess,nhesp
    READ(30,*,ERR=35,END=35) dvso, dvpo, dvsh, dvph, dvshe, dvphe
    READ(30,*,ERR=35,END=35) alpha, rh0(1), rh0(2), rhscl(1), rhscl(2)

    CLOSE(30)
   
    IF(GSKR == 'Yes' .and. mod(num, 2) /= 0) then
      WRITE(*,*) 'NUMBER OF GSK GRIDS, num, MUST BE A EVEN NUMBER'
      err0=-1
    ELSE IF(num >= nc) THEN
      WRITE(*,*) 'NUMBER OF GSK GRIDS, num, MUST BE < nc'
      err0=-1
    ELSE
      err0=0
    ENDIF

! output certain GSK run parameters for IDL visulation program
    
    OPEN(30,FILE='/home/wangy/DyFk/PLOT/GSKRUN.DAT',STATUS='UNKNOWN')

    WRITE(30,*) num
    WRITE(30,*) vsminO, vsminH, vsminHe
    WRITE(30,*) noss,nosp,nhss,nhsp,nhess,nhesp
    WRITE(30,*) dvso, dvpo, dvsh, dvph, dvshe, dvphe

    CLOSE(30)

    RETURN

35  WRITE(*,*) 'FAIL TO OPEN/READ FILE GSKRUN.DDD'
    err0=-1

END SUBROUTINE





