!----------------------------------------------------------------------
!  Move ions with mirror force, electric field and centrifugal force
!
!    Input:  efield  - parallel electric field (V/Re)
!            s     - location of particle (in Re)
!            vs    - parallel velocity of particle (in Re/s)
!            vp    - perpendicular velocity of particle (in Re/s)
!            dt    - time interval (s)
!            qmr   - charge-mass ratio
!
!    Output: s    - new location of particle
!            vs   - new parallel velocity
!            vp   - new perpendicular velocity
!
!  By Jiannan Tu, Feb 14, 2003; Modified Jan 25, 2010
!----------------------------------------------------------------------
subroutine mover(i,efield,s,vs,vp,dt,qmr)
    use inparm
    use gskparm

    integer i,k

    real(8) efield(nc) !,s1,vs1
    real(8) qmr,ef,cfs(2),gf,bf,bf1,dvs,s,vs,vp,dt,aa,mf

!-----------------------
!   qmrO = e/m_O/Re
!   qmrH = e/m_H/Re
!   gm0=gm/Re
!-----------------------
!    DATA gm0/1.5416033d-6/

! interpolate Efield, gravity, Bfield & mirror force factor to the
! particle location
    aa=(s-sg(i))*ods(i)
    ef=aa*(efield(i+1)-efield(i))+efield(i)
    gf=aa*(gm(i+1)-gm(i))+gm(i)
    bf=aa*(bfd(i+1)-bfd(i))+bfd(i)
    mf=aa*(mirrf(i+1)-mirrf(i))+mirrf(i)
    cfs(1)=aa*(cf(i+1,1)-cf(i,1))+cf(i,1)
    cfs(2)=aa*(cf(i+1,2)-cf(i,2))+cf(i,2)

! particle accel due to gravity, efiled, mirror & centrifugal force
! Veq2 & Veq included in cf(*,2) & cf(*,1), respectively
    dvs=(gf+ef*qmr+vp*vp*mf+cfs(2))/(1.0d0-cfs(1))

! advance ptcl velocity and location
    vs=vs+dvs*dt
    s=s+vs*dt
   ! if(abs(vs*dt).ge.(sg(i+1)-sg(i))) write(63,*) i,vs,qmr
    if(s.lt.sg(1).or.s.ge.sg(num+1)) then
        s=999.0d0
        return
    endif

! interpolate Bfield to new location of the particle
    k=1
    do while (sg(k+1).le.s)
        k=k+1
    enddo

    aa=(s-sg(k))*ods(k)
    bf1=aa*(bfd(k+1)-bfd(k))+bfd(k)

! calculate new vp based on conservation of magnetic moment
    vp=vp*sqrt(bf1/bf)

end subroutine mover
