!***********************************************************************
!***********************************************************************
!  SUBROUTINE read_ptcl
!  Read locations, parallel and perpendicular velocities of simulation
!  particles
!
!  INPUT:  fdir - the path & folder of files for simulation ptcls
!          sg   - array for arc distance along the flux tube
!  
!  OUTPUT: Oxygen - array for containign locations, parallel & perpend
!                   velocities of O+ simulation ptcls
!          imax_O - total number of O+ simulation ptcls
!          num_O  - array for number of O+ simulation ptcls in every grid
!                   cells
!          Hydrog - array for H+ simulation ptcls
!          imax_H - total number of H+ simulation ptcls
!          num_H  - array for H+ simulation ptcls
!
!  Author: Jiannan Tu
!  Date:   March 1, 2010
!***********************************************************************
subroutine read_ptcl(err0,n)
    USE inparm
    USE gskparm

    integer :: err0,LNBLNK,n
    character suff*6

    write(suff,'(i6)') n
    suff=adjustl(suff)

! read in O+ simu particles (location, para and perp vel)
    open(11,file='/home/wangy/DyFk/INITIAL/Oxygen.'//suff(:LNBLNK(suff)),status='old', &
         err=215,form='unformatted')

    read(11,end=115,err=215) Oxygen

115 close(11)

! read in H+ simu particles (location, para and perp vel)
    open(11,file='/home/wangy/DyFk/INITIAL/Hydrog.'//suff(:LNBLNK(suff)),status='old', &
         err=215,form='unformatted')

    read(11,end=125,err=215) Hydrog

125 close(11)

! read in He+ simu particles (location, para and perp vel)
    open(11,file='/home/wangy/DyFk/INITIAL/Helium.'//suff(:LNBLNK(suff)),status='old', &
         err=215,form='unformatted')

    read(11,end=135,err=215) Helium

135 close(11)

! get values of q if continuous run & successfully loaded ptcls
    if(CRUN=='Yes'.or.err0==0) then
      open(11,file='/home/wangy/DyFk/INITIAL/gsk_crun.'//suff(:LNBLNK(suff)),status='old',&
            form='formatted')
      read(11,*) num,scalg
      read(11,*) imax_O,imax_H,imax_He
      read(11,*) rep_O,rep_H,rep_He
      read(11,*) qval
      close(11)
    endif
    
    return

! exception handling
215 write(*,*) 'rank',n,'Fail to input particles from files. Start a completely new run'
    err0=-1

end subroutine
