!-----------------------------------------------------------------------
!  Calculate ion velocity distribution in several specified altitude
!  regions
!
!    Input:  OHgen - array of location (in Re), para and perp velocities
!                    (in Re/s) of sim particles
!            imax  - total number of simulation particles
!            rep   - number of real particle in a simulation particle
!            dvs   - parallel velocity interval (in km/s)
!            dvp   - perpendicular velocity interval in (km/s)
!            vismin - velocity limit for resulting distribution (km/s)
!            nss   - number of grid in parallel velocity axis
!            nsp   - number of gird in perpendicular velocity axis
!
!    Output: vsvp_s - velocity distribution f(vs, vp)
!
!  Jiannan Tu, Feb 14, 2003. Modified 4/5/2010
!-----------------------------------------------------------------------
subroutine velocity_distr(OHgen,imax,dvs,dvp,vsmin,nss,nsp,vsvp_s)
    use gskparm

    integer nss,nsp
      
    real(8) OHgen(np,3),vsvp_s(nss,nsp)
    real(8) dvs,dvp,vsmin,Re,s,vs,vp,aii,dai1,dai2,upaii
    real(8) vsminr,dvsr,dvpr
    integer imax,i,j,ij,ii,jj,it

    data Re/6371.2d0/ ! Re in km because vsmin, dvs & dvp in km/s

! normalized by Re
    vsminr=vsmin/Re
    dvsr=dvs/Re
    dvpr=dvp/Re

    upaii=float(nss)+1.5d0

    do i=1,imax
      s=OHgen(i,1)
      vs=OHgen(i,2)
      vp=OHgen(i,3)
      aii=(vs-vsminr)/dvsr+1.0d0
      jj=int(vp/dvpr+1.0d0)

      if(aii.gt.0.0d0.and.aii.le.upaii) then
        ii=int(aii)
        dai1=aii-float(ii)-0.5d0
        if (dai1.lt.0.0d0) then
          it=ii-1
          dai1=abs(dai1)
        else
          it=ii+1
        endif
        dai2=1.0d0-dai1

        if(ii.ge.1.and.ii.le.nss.and.jj.ge.1.and.jj.le.nsp) &
           vsvp_s(ii,jj)=vsvp_s(ii,jj)+dai2

        if(it.ge.1.and.it.le.nss.and.jj.ge.1.and.jj.le.nsp) &
           vsvp_s(it,jj)=vsvp_s(it,jj)+dai1
      endif
    enddo
end subroutine
