!***********************************************************************
! Long period (>2 x 10^18) random number generator of L'Ecuyer with
! Bays-Durham shuffle and added safeguards. Returns a uniform random
! deviate betwee 0.0 and 1.0 (exclusive of the endpoint values). Call
! with idum a negative integer to initialize; thereafter, do not alter
! idum between successive deviates in a sequence. RNMX should approximate
! the longest floating values that is less than 1.
!
! Code from Numerical Recipes, 2nd eds, W. H. Press and S. A. Teukolsky
! 1992.
!***********************************************************************
real*8 function ran2(idum)
    integer idum,IM1,IM2,IMM1,IA1,IA2,IQ1,IQ2,IR1,IR2,NTAB,NDIV
    real*8 AM,EPS,RNMX
    parameter (IM1=2147483563,IM2=2147483399,AM=1.d0/dfloat(IM1)) !4.656613057e-10)
    parameter (IMM1=2147483562,IA1=40014,IA2=40692,IQ1=53668,IQ2=52774)
    parameter (IR1=12211,IR2=3791,NTAB=32,NDIV=67108862,EPS=1.2d-7)
    parameter (RNMX=0.9999999d0)

    integer idum2,j,k,iv(NTAB),iy
    save iv,iy,idum2
    data idum2/123456789/,iv/NTAB*0/,iy/0/

    if(idum.le.0) then           !Initialize
        idum=max(-idum,1)        !Be sure to prevent idum=0
        idum2=idum

        do j=NTAB+8,1,-1         !Load the shuffle table (aft 8 warm-ups)
            k=idum/IQ1
            idum=IA1*(idum-k*IQ1)-k*IR1
            if(idum.lt.0) idum=idum+IM1
            if(j.le.NTAB) iv(j)=idum
        enddo

        iy=iv(1)
    endif

    k=idum/IQ1                      !Start here when not initializing
    idum=IA1*(idum-k*IQ1)-k*IR1     !Compute idum=mod(IA1*idum,IM1) without
    if(idum.lt.0) idum=idum+IM1     !  overflows by Schrage's method
    k=idum2/IQ2
    idum2=IA2*(idum2-k*IQ2)-k*IR2   !Compute idum2=mod(IA2*idum2,IM2) likewise
    if(idum2.lt.0) idum2=idum2+IM2
    j=1+iy/NDIV                     !Will be in the range 1:NTAB
    iy=iv(j)-idum2                  !Here idum is shuffled, idum & idum2 are
    iv(j)=idum                      !  combined to generate output
    if(iy.lt.1) iy=iy+IMM1
    ran2=min(AM*iy,RNMX)            !Because users don't expect endpoint value

END FUNCTION
